@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 2002 Hayato Fujiwara <h_fujiwara@users.sourceforge.net>
-Copyright (C) 2010-2016 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2010-2018 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,13 +22,22 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 // PKG_DEL: autoload ("pserver", "parallel_interface.oct", "remove");
 
 #include <octave/oct.h>
-#include <octave/symtab.h>
+
+#include "config.h"
+
 #include <octave/oct-env.h>
 #include <octave/file-io.h>
 #include <octave/sighandlers.h>
 #include <octave/parse.h>
 #include <octave/cmd-edit.h>
+#ifdef HAVE_OCTAVE_INTERPRETER_H
+// >= octave-4.2
+#include <octave/octave.h>
+#include <octave/interpreter.h>
+#else
+// <= octave-4.0..
 #include <octave/toplev.h>
+#endif
 
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -47,8 +56,6 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 
 #include "parallel-gnutls.h"
 
-#include "config.h"
-
 // Octave > 3.2.4 does not have these in a header file, but in
 // sighandlers.cc, and uses gnulib:: for these. So this is copied from
 // Octave-3.2.4.
@@ -78,7 +85,7 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 static
 int assert_file (std::string &path)
 {
-  file_stat stat (path);
+  OCTAVE__SYS__FILE_STAT stat (path);
 
   if (! stat.is_reg ())
     return -1;
@@ -140,7 +147,7 @@ reval_loop (octave_parallel_stream &cstr)
 
       bool err;
       int p_err;
-      SET_ERR (eval_string (s, false, p_err, 0), err);
+      SET_ERR (OCTAVE__EVAL_STRING (s, false, p_err, 0), err);
       dsprintf ("reval loop, after evaluating string\n");
 
       uint32_t nl = 0;
@@ -389,10 +396,10 @@ the swap device even after shutdown of the application.\n\
     warning ("no TLS used, option 'auth_file' has no effect");
 
   // initialize exit function
-  feval ("__pserver_exit__", octave_value (hostname), 0);
+  OCTAVE__FEVAL ("__pserver_exit__", octave_value (hostname), 0);
 
   int tp;
-  if (tp = fork ())
+  if ((tp = fork ()))
     {
       if (tp == -1)
         error ("could not fork");
@@ -400,7 +407,7 @@ the swap device even after shutdown of the application.\n\
     }
 
   // register exit function
-  feval ("atexit", octave_value ("__pserver_exit__"), 0);
+  OCTAVE__FEVAL ("atexit", octave_value ("__pserver_exit__"), 0);
 
   /* write pidfile and mark for deletion. */
   if (check_and_write_pidfile (pidname.c_str (), getpid ()))
@@ -408,25 +415,25 @@ the swap device even after shutdown of the application.\n\
       std::cerr << "octave: " << hostname.c_str ()
                 << ": can't write pidfile, server possibly already running"
                 << std::endl;
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
   mark_for_deletion (pidname.c_str ());
   std::cout << pidname.c_str () << std::endl;
 
   // avoid dumping octave_core if killed by a signal
-  feval ("sigterm_dumps_octave_core", octave_value (0), 0);
-  feval ("sighup_dumps_octave_core", octave_value (0), 0);
+  OCTAVE__FEVAL ("sigterm_dumps_octave_core", octave_value (0), 0);
+  OCTAVE__FEVAL ("sighup_dumps_octave_core", octave_value (0), 0);
 
   /* Redirect stdin and stdout to /dev/null. */
   if (! freopen ("/dev/null", "r", stdin))
     {
       perror ("freopen");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
   if (! freopen ("/dev/null", "w", stdout))
     {
       perror ("freopen");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
   std::string errname ("/tmp/octave_error-");
@@ -439,13 +446,13 @@ the swap device even after shutdown of the application.\n\
       if (rename (errname.c_str (), bakname.c_str ()))
         {
           perror ("rename");
-          clean_up_and_exit (1);
+          exit (1); // clean_up_and_exit (1);
         }
     }
   if (! freopen (errname.c_str (), "w", stderr))
     {
       perror ("freopen");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     } 
 
   struct addrinfo *ai, hints;
@@ -458,53 +465,53 @@ the swap device even after shutdown of the application.\n\
   if (getaddrinfo (NULL, "12502", &hints, &ai))
     {
       perror ("getaddrinfo");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
   int sock = socket (PF_INET, SOCK_STREAM, 0);
   if (sock == -1)
     {
       perror ("socket");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
   if (bind (sock, ai->ai_addr, ai->ai_addrlen))
     {
       perror ("bind");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
 
   if (listen (sock, SOMAXCONN))
     {
       perror ("listen");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
 
   if (getaddrinfo (NULL, "12501", &hints, &ai))
     {
       perror ("getaddrinfo");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
   int dsock = socket (PF_INET, SOCK_STREAM, 0);
   if (dsock == -1)
     {
       perror ("socket");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
   if (bind (dsock, ai->ai_addr, ai->ai_addrlen))
     {
       perror ("bind");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
   if (listen (dsock, SOMAXCONN))
     {
       perror ("listen");
-      clean_up_and_exit (1);
+      exit (1); // clean_up_and_exit (1);
     }
 
   freeaddrinfo (ai);
@@ -518,7 +525,7 @@ the swap device even after shutdown of the application.\n\
       if (asock == -1)
         {
           perror ("accept");
-          clean_up_and_exit (1);
+          exit (1); // clean_up_and_exit (1);
         }
       dsprintf ("%i: accepted command stream\n", server_no);
       /* Normal production daemon.  Fork, and have the child process
@@ -540,12 +547,12 @@ the swap device even after shutdown of the application.\n\
       if ((pid = fork ()) == -1)
         {
           perror ("fork");
-          clean_up_and_exit (1);
+          exit (1); // clean_up_and_exit (1);
         }
       else if (pid == 0) 
         {
           int tp;
-          if (tp = fork ())
+          if ((tp = fork ()))
             {
               if (tp == -1)
                 {
@@ -604,27 +611,34 @@ the swap device even after shutdown of the application.\n\
               // generate credentials
               if (! cafile.length ())
                 {
+#ifdef HAVE_OCTAVE_CONFIG_FCNS
+                  std::string octave_home = octave::config::octave_home ();
+#else
                   extern std::string Voctave_home;
-                  cafile = Voctave_home + file_ops::dir_sep_str () + "share" +
-                    file_ops::dir_sep_str () + "octave" +
-                    file_ops::dir_sep_str () + "parallel-srp-data" +
-                    file_ops::dir_sep_str () + "server" +
-                    file_ops::dir_sep_str () + "passwd";
+                  std::string octave_home = Voctave_home;
+#endif
+                  cafile = octave_home +
+                    OCTAVE__SYS__FILE_OPS::dir_sep_str () + "share" +
+                    OCTAVE__SYS__FILE_OPS::dir_sep_str () + "octave" +
+                    OCTAVE__SYS__FILE_OPS::dir_sep_str () +
+                    "parallel-srp-data" +
+                    OCTAVE__SYS__FILE_OPS::dir_sep_str () + "server" +
+                    OCTAVE__SYS__FILE_OPS::dir_sep_str () + "passwd";
                   if (assert_file (cafile))
                     {
                       octave_value_list f_args (1);
                       f_args(0) = octave_value ("prefix");
                       octave_value_list f_ret;
-                      CHECK_ERROR_EXIT1 (f_ret = feval ("pkg", f_args, 1),
+                      CHECK_ERROR_EXIT1 (f_ret = OCTAVE__FEVAL ("pkg", f_args, 1),
                                          "%s: could not get prefix from pkg",
                                          fname.c_str ());
                       CHECK_ERROR_EXIT1 (cafile = f_ret(0).string_value (),
                                          "%s: could not convert output of pkg ('prefix') to string)",
                                          fname.c_str ());
-                      cafile = cafile + file_ops::dir_sep_str () +
-                        "parallel-srp-data" + file_ops::dir_sep_str () +
-                        file_ops::dir_sep_str () + "server" +
-                        file_ops::dir_sep_str () + "passwd";
+                      cafile = cafile + OCTAVE__SYS__FILE_OPS::dir_sep_str () +
+                        "parallel-srp-data" +
+                        OCTAVE__SYS__FILE_OPS::dir_sep_str () + "server" +
+                        OCTAVE__SYS__FILE_OPS::dir_sep_str () + "passwd";
                       if (assert_file (cafile))
                         {
                           error ("%s: no regular file found at default password file paths",
@@ -669,7 +683,7 @@ the swap device even after shutdown of the application.\n\
             }
           else
 #endif
-            cmd_strb = new octave_parallel_socket_streambuf (asock);
+            cmd_strb = new octave_parallel_socket_streambuf (asock, true);
           octave_parallel_stream cmd_str (cmd_strb);
           if (! cmd_str.good ())
             {
@@ -797,7 +811,6 @@ the swap device even after shutdown of the application.\n\
               int not_connected = 1;
               for (int j = 0; j < N_CONNECT_RETRIES; j++)
                 {
-                  int32_t result;
                   struct sockaddr_in rem_addr;
                   socklen_t addrlen = sizeof (rem_addr);
                   int adsock = accept (dsock, (sockaddr *) &rem_addr, &addrlen);
@@ -860,7 +873,7 @@ the swap device even after shutdown of the application.\n\
 #endif
                     conn->insert_data_stream
                       (new octave_parallel_stream
-                       (new octave_parallel_socket_streambuf (adsock)));
+                       (new octave_parallel_socket_streambuf (adsock, true)));
 
                   __sockg.__release ();
 
@@ -908,8 +921,8 @@ the swap device even after shutdown of the application.\n\
                     {
                       conn->get_data_stream ()->network_send_4byteint (0);
                       int err = conn->connection_read_header ();
-                      write_header (conn->get_data_stream ()->get_ostream (),
-                                    LS_BINARY);
+                      minimal_write_header
+                        (conn->get_data_stream ()->get_ostream ());
                       if (err || ! conn->get_data_stream ()->good ())
                         {
                           error ("communication error in initialization");
@@ -1021,7 +1034,7 @@ the swap device even after shutdown of the application.\n\
 #endif
                         conn->insert_data_stream
                           (new octave_parallel_stream
-                           (new octave_parallel_socket_streambuf (dsock)));
+                           (new octave_parallel_socket_streambuf (dsock, false)));
                       __sockg.__release ();
                       if (! conn->get_data_stream ()->good ())
                         {
@@ -1065,9 +1078,8 @@ the swap device even after shutdown of the application.\n\
                         }
                       else
                         {
-                          write_header
-                            (conn->get_data_stream ()->get_ostream (),
-                             LS_BINARY);
+                          minimal_write_header
+                            (conn->get_data_stream ()->get_ostream ());
                           if (conn->connection_read_header () ||
                               ! conn->get_data_stream ()->good ())
                             {
@@ -1102,10 +1114,14 @@ the swap device even after shutdown of the application.\n\
 
           __pwg.__free ();
 
+#ifdef HAVE_OCTAVE_INTERPRETER_H
+          OCTAVE__INTERPRETER::the_interpreter () -> interactive (false);
+#else
           interactive = false;
+#endif
 
           // install 'sockets' as Octave variable
-          symbol_table::assign ("sockets", sockets);
+          OCTAVE__INTERPRETER__SYMBOL_TABLE__ASSIGN ("sockets", sockets);
           dsprintf ("'sockets' installed\n");
 
           int cd_ok = octave_env::chdir (directory.c_str ());
@@ -1127,7 +1143,7 @@ the swap device even after shutdown of the application.\n\
 
       // FIXME: forgotten how this is organized in Octave, find it out
       // again and at least make a comment which better explains it.
-      octave_child_list::insert (pid, pserver_child_event_handler);
+      OCTAVE_CHILD_LIST insert (pid, pserver_child_event_handler);
 
       RESTORE_SIGNALS (oset);
 
@@ -1137,5 +1153,7 @@ the swap device even after shutdown of the application.\n\
   // This code is currently not reached since the server parent
   // process can only be killed by a signal.
   close (sock);
-  clean_up_and_exit (1);
+  exit (1); // clean_up_and_exit (1);
+  // Silence compiler warning.
+  exit (0);
 }
